
/**
 * 
 * @author Lorenz Schauer
 * 
 * Implementierende Klasse der abstrakten Klasse HoersaalAbstrakt
 *
 */
public class Hoersaal extends HoersaalAbstrakt {
		
	//Konstrukotr belegt die Instanzvariablen
	public Hoersaal(int cap){
		
		MAX_CAPACITY = cap; 
		anzahlStudenten=0;
		professorKommt = false;
		professorIstImSaal = false;
	}
	
	
	/**
	 * Student betritt Hoersaal nur, wenn er frei ist, der Professor nicht kommt und der Professor nicht im Hrsaal ist
	 */
	public synchronized void student_betritt() throws InterruptedException{
		
		while(anzahlStudenten>=MAX_CAPACITY || professorKommt || professorIstImSaal)
			wait();
		
		anzahlStudenten++;
		
		System.out.println(Thread.currentThread().getName()+" betritt Hoersaal. ("+anzahlStudenten+" anwesend)");

	}
		
	/**
	 * Student kann den Hrsaal immer verlassen und benachrichtigt dann den Professor bzw. die Studenten, die drauen auf Einlass warten
	 */
	public synchronized void student_verlaesst(){
		
		anzahlStudenten--;
		
		System.out.println(Thread.currentThread().getName()+" verlaesst Hoersaal. ("+anzahlStudenten+" anwesend)");
		notifyAll();
		
	}
	
	/**
	 * Der Professor betritt nur, wenn kein Student mehr im Hrsaal ist. Sonst wartet er solange
	 */
	public synchronized void professor_betritt() throws InterruptedException{
		
		while(anzahlStudenten>0)
			wait();
		System.out.println(Thread.currentThread().getName()+" betritt Hoersaal. ("+anzahlStudenten+" anwesend)");
		this.professorKommt=false; 
		this.professorIstImSaal=true;

	}
	
	/**
	 * Der Professor kann den Hrsaal einfach verlassen und benachrichtig wieder die Studenten, die drauen schon auf Einlass warten
	 */
	public synchronized void professor_verlaesst(){
		
		System.out.println(Thread.currentThread().getName()+" verlaesst Hoersaal. ("+anzahlStudenten+" anwesend)");
		this.professorIstImSaal=false;
		
		notifyAll();	
	}
	
	/**
	 * Methode zum setzen des Flags, damit angezeigt wird, dass der Professor im Anmarsch ist und kein neuer Student mehr den Hrsaal betreten soll. 
	 * Muss nicht synchronisiert sein!
	 */
	public void professorKommt(){
		this.professorKommt=true;
		System.out.println("--> Professor im Anmarsch...");
	}
}
